@@ -1,13 +1,12 @@
 use std::collections::HashMap;
 
-use crate::ast::*;
+use crate::{asm::msize, ast::*};
 use thiserror::Error;
 
-const STACK_SIZE: usize = 256;
-const RAM_SIZE: usize = 64_000;
-
 type LabelMap<'a> = HashMap<Label<'a>, usize>;
 
+pub type Computer64<'a> = Computer<'a, 64_000, 256>;
+
 macro_rules! expect {
     ($expression:expr, $status:expr) => {
         match $expression {
@@ -155,26 +154,29 @@ impl From<std::cmp::Ordering> for Comparison {
     }
 }
 
-trait Read {
-    fn read(&self, memory: &Memory) -> u16;
+/// The `Read` trait allows for making instructions generic over data sources.
+trait Read<const R: usize, const S: usize> {
+    fn read(&self, memory: &Memory<R, S>) -> msize;
 }
 
-trait Write: Read {
-    fn write(&self, memory: &mut Memory, value: u16);
+/// The `Write` trait allows for making instructions generic over data stores.
+trait Write<const R: usize, const S: usize>: Read<R, S> {
+    fn write(&self, memory: &mut Memory<R, S>, value: msize);
 }
 
-trait AsRead {
-    fn as_read(&self) -> Option<&dyn Read>;
+/// The `AsRead` trait allows for partialially readable types to downcast into trait objects.
+trait AsRead<const R: usize, const S: usize> {
+    fn as_read(&self) -> Option<&dyn Read<R, S>>;
 }
 
-impl AsRead for Option<&Argument<'_>> {
-    fn as_read(&self) -> Option<&dyn Read> {
+impl<const R: usize, const S: usize> AsRead<R, S> for Option<&Argument<'_>> {
+    fn as_read(&self) -> Option<&dyn Read<R, S>> {
         self.and_then(|inside| inside.as_read())
     }
 }
 
-impl AsRead for Argument<'_> {
-    fn as_read(&self) -> Option<&dyn Read> {
+impl<const R: usize, const S: usize> AsRead<R, S> for Argument<'_> {
+    fn as_read(&self) -> Option<&dyn Read<R, S>> {
         Some(match self {
             Argument::Register(register) => register,
             Argument::Address(address) => address,
@@ -184,18 +186,19 @@ impl AsRead for Argument<'_> {
     }
 }
 
-trait AsWrite {
-    fn as_write(&self) -> Option<&dyn Write>;
+/// The `AsWrite` trait allows for partialially readable types to downcast into trait objects.
+trait AsWrite<const R: usize, const S: usize> {
+    fn as_write(&self) -> Option<&dyn Write<R, S>>;
 }
 
-impl AsWrite for Option<&Argument<'_>> {
-    fn as_write(&self) -> Option<&dyn Write> {
+impl<const R: usize, const S: usize> AsWrite<R, S> for Option<&Argument<'_>> {
+    fn as_write(&self) -> Option<&dyn Write<R, S>> {
         self.and_then(|inside| inside.as_write())
     }
 }
 
-impl AsWrite for Argument<'_> {
-    fn as_write(&self) -> Option<&dyn Write> {
+impl<const R: usize, const S: usize> AsWrite<R, S> for Argument<'_> {
+    fn as_write(&self) -> Option<&dyn Write<R, S>> {
         Some(match self {
             Argument::Register(register) => register,
             Argument::Address(address) => address,
@@ -204,20 +207,20 @@ impl AsWrite for Argument<'_> {
     }
 }
 
-impl Read for IntLiteral {
-    fn read(&self, _: &Memory) -> u16 {
+impl<const R: usize, const S: usize> Read<R, S> for IntLiteral {
+    fn read(&self, _: &Memory<R, S>) -> msize {
         self.value()
     }
 }
 
-impl Read for Register {
-    fn read(&self, memory: &Memory) -> u16 {
+impl<const R: usize, const S: usize> Read<R, S> for Register {
+    fn read(&self, memory: &Memory<R, S>) -> msize {
         memory.registers[self.index]
     }
 }
 
-impl Read for Address {
-    fn read(&self, memory: &Memory) -> u16 {
+impl<const R: usize, const S: usize> Read<R, S> for Address {
+    fn read(&self, memory: &Memory<R, S>) -> msize {
         let address = match self {
             literal @ Address::IntLiteral(..) => literal.read(memory),
             Address::Register(register) => register.read(memory),
@@ -226,14 +229,14 @@ impl Read for Address {
     }
 }
 
-impl Write for Register {
-    fn write(&self, memory: &mut Memory, value: u16) {
+impl<const R: usize, const S: usize> Write<R, S> for Register {
+    fn write(&self, memory: &mut Memory<R, S>, value: msize) {
         memory.registers[self.index] = value;
     }
 }
 
-impl Write for Address {
-    fn write(&self, memory: &mut Memory, value: u16) {
+impl<const R: usize, const S: usize> Write<R, S> for Address {
+    fn write(&self, memory: &mut Memory<R, S>, value: msize) {
         let address = match self {
             literal @ Address::IntLiteral(..) => literal.read(memory),
             Address::Register(register) => register.read(memory),
@@ -242,14 +245,15 @@ impl Write for Address {
     }
 }
 
-pub struct Stack {
-    content: [u16; STACK_SIZE],
+/// Statically allocated stack memory.
+pub struct Stack<const S: usize> {
+    content: [msize; S],
     pointer: usize,
 }
 
-impl Default for Stack {
+impl<const S: usize> Default for Stack<S> {
     fn default() -> Self {
-        let content = [0; STACK_SIZE];
+        let content = [0; S];
         Self {
             content,
             pointer: content.len(),
@@ -257,8 +261,9 @@ impl Default for Stack {
     }
 }
 
-impl Stack {
-    pub fn push(&mut self, value: u16) -> Result<(), Error> {
+impl<const S: usize> Stack<S> {
+    /// Push an element to the end of the stack.
+    pub fn push(&mut self, value: msize) -> Result<(), Error> {
         if self.pointer > 0 {
             self.pointer -= 1;
             self.content[self.pointer] = value;
@@ -268,7 +273,8 @@ impl Stack {
         }
     }
 
-    pub fn pop(&mut self) -> Result<u16, Error> {
+    /// Pop an element from the end of the stack.
+    pub fn pop(&mut self) -> Result<msize, Error> {
         if self.pointer < self.content.len() {
             let value = self.content[self.pointer];
             self.pointer += 1;
@@ -279,26 +285,28 @@ impl Stack {
     }
 }
 
-pub struct Memory {
-    accumulator: u16,
-    registers: [u16; 16],
-    stack: Stack,
-    ram: [u16; RAM_SIZE],
+/// Memory component of [`Computer`].
+pub struct Memory<const R: usize, const S: usize> {
+    accumulator: msize,
+    registers: [msize; 16],
+    stack: Stack<S>,
+    ram: [msize; R],
     comparitor: Comparison,
 }
 
-impl Default for Memory {
+impl<const R: usize, const S: usize> Default for Memory<R, S> {
     fn default() -> Self {
         Self {
             accumulator: Default::default(),
             registers: Default::default(),
             stack: Default::default(),
-            ram: [0; RAM_SIZE],
+            ram: [0; R],
             comparitor: Default::default(),
         }
     }
 }
 
+/// Data buffer for handling computer I/O.
 #[derive(Default)]
 pub struct BufStream<T> {
     content: Vec<T>,
@@ -313,23 +321,27 @@ impl<T: std::fmt::Debug> std::fmt::Debug for BufStream<T> {
 }
 
 impl<T> BufStream<T> {
+    /// Consume a single value from the output.
     pub fn read(&mut self) -> Option<T> {
         let mut value = (!self.content.is_empty()).then(|| self.content.remove(0));
         if let Some(provider) = &self.provider {
-            value = value.or(provider());
+            value = value.or_else(provider);
         }
         value
     }
 
+    /// Consume entire output buffer.
     pub fn read_all(&mut self) -> Vec<T> {
         self.content.drain(..).collect()
     }
 
+    /// Extend input with a single value.
     pub fn write(&mut self, value: T) {
         self.subscribers.iter().for_each(|f| f(&value));
         self.content.push(value);
     }
 
+    /// Extend input with contents of iterator.
     pub fn write_iter(&mut self, iter: impl IntoIterator<Item = T>) {
         let mut new_content: Vec<T> = iter.into_iter().collect();
         self.subscribers
@@ -338,35 +350,35 @@ impl<T> BufStream<T> {
         self.content.append(&mut new_content)
     }
 
+    /// Subscribe to output events with a non-consuming listener callback.
     pub fn subscribe(&mut self, callback: impl Fn(&T) + 'static) {
         self.subscribers.push(Box::new(callback))
     }
 
+    /// Provide a source of input to fall back on.
     pub fn provide(&mut self, provider: impl Fn() -> Option<T> + 'static) {
         self.provider = Some(Box::new(provider));
     }
 }
 
+/// Virtual computer for executing M64 ASM programs.
 #[derive(Default)]
-pub struct Computer<'a> {
+pub struct Computer<'a, const R: usize, const S: usize> {
     program_counter: usize,
     labels: LabelMap<'a>,
     statements: Vec<Statement<'a>>,
-    memory: Memory,
+    memory: Memory<R, S>,
     status: Status,
-    output: BufStream<u16>,
-    input: BufStream<u16>,
+    output: BufStream<msize>,
+    input: BufStream<msize>,
 }
 
-impl<'a> Computer<'a> {
-    pub fn status(&self) -> Status {
-        self.status
-    }
-
-    pub fn program_counter(&self) -> usize {
-        self.program_counter
-    }
-
+impl<'a, const R: usize, const S: usize> Computer<'a, R, S> {
+    /// Loads new program into computer, overwriting the old one.
+    ///
+    /// This does not reset the computer's state apart from the program counter. Either have your
+    /// program intitialize the memory it wishes to use or call [`Computer::reset`] before
+    /// executing.
     pub fn load_program(&mut self, program: File<'a>) {
         self.statements = program.statements;
         self.labels = self
@@ -382,11 +394,92 @@ impl<'a> Computer<'a> {
             })
             .collect();
 
-        if let Some(&start) = self.labels.get(&Label { value: ".start" }) {
-            self.program_counter = start;
+        self.program_counter = self
+            .labels
+            .get(&Label { value: ".start" })
+            .copied()
+            .unwrap_or_default();
+    }
+
+    /// Chainable variant of [`Computer::load_program`].
+    pub fn with_program(mut self, program: File<'a>) -> Self {
+        self.load_program(program);
+        self
+    }
+
+    /// Reset's computer's memory
+    pub fn reset(&mut self) {
+        self.memory = Memory::default();
+        self.program_counter = self
+            .labels
+            .get(&Label { value: ".start" })
+            .copied()
+            .unwrap_or_default();
+    }
+
+    /// Chainable variant of [`Computer::reset`].
+    pub fn with_reset(mut self) -> Self {
+        self.reset();
+        self
+    }
+
+    /// Ge the current status of the computer.
+    pub fn status(&self) -> Status {
+        self.status
+    }
+
+    /// Get the current value of the program counter.
+    pub fn program_counter(&self) -> usize {
+        self.program_counter
+    }
+
+    /// Execute until program terminates.
+    pub fn execute(&mut self) -> Status {
+        loop {
+            let status = self.step();
+            if status.is_stopped() {
+                break status;
+            }
         }
     }
 
+    /// Execute until YLD instruction or program terminates.
+    pub fn execute_until_yield(&mut self) -> Status {
+        while self.step().is_ready() {}
+        self.status
+    }
+
+    /// Consume a single value from the output.
+    pub fn read(&mut self) -> Option<msize> {
+        self.output.read()
+    }
+
+    /// Consume entire output buffer.
+    pub fn read_all(&mut self) -> Vec<msize> {
+        self.output.read_all()
+    }
+
+    /// Extend input with a single value.
+    pub fn write(&mut self, value: msize) {
+        self.input.write(value)
+    }
+
+    /// Extend input with contents of iterator.
+    pub fn write_iter(&mut self, iter: impl IntoIterator<Item = msize>) {
+        self.input.write_iter(iter)
+    }
+
+    /// Subscribe to output events with a non-consuming listener callback.
+    pub fn subscribe(&mut self, callback: impl Fn(&msize) + 'static) {
+        self.output.subscribe(callback)
+    }
+
+    /// Provide a source of input to fall back on.
+    pub fn provide(&mut self, provider: impl Fn() -> Option<msize> + 'static) {
+        self.input.provide(provider)
+    }
+
+    /// Execute a single instruction.
     pub fn step(&mut self) -> Status {
         if self.status.is_yield() {
             self.status = Status::Ready;
@@ -560,14 +653,16 @@ impl<'a> Computer<'a> {
                             if expect!(
                                 self.memory
                                     .stack
-                                    .push((self.program_counter & 0xFFFF) as u16),
+                                    .push((self.program_counter & 0xFFFF) as msize),
                                 self.status
                             )
                             .is_some()
                             {
                                 // upper 16 bits of PC
                                 if expect!(
-                                    self.memory.stack.push((self.program_counter >> 16) as u16),
+                                    self.memory
+                                        .stack
+                                        .push((self.program_counter >> 16) as msize),
                                     self.status
                                 )
                                 .is_some()
@@ -693,42 +788,4 @@ impl<'a> Computer<'a> {
 
         self.status
     }
-
-    pub fn execute(&mut self) -> Status {
-        loop {
-            let status = self.step();
-            if status.is_stopped() {
-                break status;
-            }
-        }
-    }
-
-    pub fn execute_until_yield(&mut self) -> Status {
-        while self.step().is_ready() {}
-        self.status
-    }
-
-    pub fn read(&mut self) -> Option<u16> {
-        self.output.read()
-    }
-
-    pub fn read_all(&mut self) -> Vec<u16> {
-        self.output.read_all()
-    }
-
-    pub fn subscribe(&mut self, callback: impl Fn(&u16) + 'static) {
-        self.output.subscribe(callback)
-    }
-
-    pub fn write(&mut self, value: u16) {
-        self.input.write(value)
-    }
-
-    pub fn write_iter(&mut self, iter: impl IntoIterator<Item = u16>) {
-        self.input.write_iter(iter)
-    }
-
-    pub fn provide(&mut self, provider: impl Fn() -> Option<u16> + 'static) {
-        self.input.provide(provider)
-    }
 }
